% Copyright (c) 2011 Joris Degroote, Majid Hojjat, Electra Stavropoulou,
% Roland Wüchner, Kai-Uwe Bletzinger
%
% Licensed under the Non-Profit Open Software License version 3.0
% (NPOSL-3.0). You should have received a copy of the NPOSL-3.0 along with 
% this file. If not, see <http://www.opensource.org/licenses/NPOSL-3.0>
%
% Animation of the result.
    
classdef Animator < handle
    properties (SetAccess=private)
        n;
        m_e;
        time;
        r_history;
        p_history;
        figure_r_animation;
        handle_r_animation;
        figure_r_history;
        handle_r_history;
        figure_p_animation;
        handle_p_animation;
        figure_p_history;
        handle_p_history;
    end
    properties (SetAccess=immutable)
        dt;
        delay;
        graphics;
    end
    methods
        function A=Animator(dt,dz,m_e,n_e,delay,graphics)
            A.n=0;
            A.m_e=m_e;
            A.time=0;
            A.r_history=0;
            A.p_history=0;
            
            if (graphics)
                scrsz=get(0,'ScreenSize');
                color='k';
                linewidth=2.0;
                min_r_value=0.0;
                max_r_value=0.0008;
                min_p_value=0.0;
                max_p_value=16000.0;
            
                A.figure_r_animation=figure('OuterPosition',...
                    [1 scrsz(4)/2 scrsz(3)/2 scrsz(4)/2]);
                A.handle_r_animation=plot(dz/2:dz:m_e*dz,zeros(1,m_e), ...
                    'EraseMode','xor','Color',color,'LineWidth',linewidth);
                ylim([min_r_value max_r_value]);
                xlabel('z [m]');
                ylabel('r [m]');
            
                A.figure_r_history=figure('OuterPosition',...
                    [scrsz(3)/2 scrsz(4)/2 scrsz(3)/2 scrsz(4)/2]);
                A.handle_r_history=plot(A.time,A.r_history,'EraseMode',...
                    'xor','Color',color,'LineWidth',linewidth);
                xlim([0 dt*n_e]);
                ylim([min_r_value max_r_value]);
                xlabel('t [s]');
                ylabel('r [m]');
                
                A.figure_p_animation=figure('OuterPosition',...
                    [1 0 scrsz(3)/2 scrsz(4)/2]);
                A.handle_p_animation=plot(dz/2:dz:m_e*dz,zeros(1,m_e), ...
                    'EraseMode','xor','Color',color,'LineWidth',linewidth);
                ylim([min_p_value max_p_value]);
                xlabel('z [m]');
                ylabel('p [Pa]');
            
                A.figure_p_history=figure('OuterPosition',...
                    [scrsz(3)/2 0 scrsz(3)/2 scrsz(4)/2]);
                A.handle_p_history=plot(A.time,A.p_history,'EraseMode',...
                    'xor','Color',color,'LineWidth',linewidth);
                xlim([0 dt*n_e]);
                ylim([min_p_value max_p_value]);
                xlabel('t [s]');
                ylabel('p [Pa]');
            end
            
            A.dt=dt;
            A.delay=delay;
            A.graphics=graphics;
        end
        function delete(A)
            if (A.graphics)
                close(A.figure_r_animation);
                close(A.figure_r_history);
                close(A.figure_p_animation);
                close(A.figure_p_history);
            end
        end
        function add(A,x_f,x_s)
            A.time=[A.time; A.n*A.dt];
            A.r_history=[A.r_history; x_s(floor(A.m_e/2))];
            A.p_history=[A.p_history; x_f(4+A.m_e+floor(A.m_e/2))];
            if (A.graphics)
                set(A.handle_r_animation,'YData',x_s(1:A.m_e));
                set(A.handle_r_history,'XData',A.time,'YData',A.r_history);
                set(A.handle_p_animation,'YData',x_f(5+A.m_e:4+2*A.m_e));
                set(A.handle_p_history,'XData',A.time,'YData',A.p_history);
                drawnow;
            end
        end
        function increase_time(A)
            A.n=A.n+1;
            if (A.graphics)
                pause(A.delay);
            end
        end
        function decrease_time(A)
            A.n=A.n-1;
        end
    end
end