% Copyright (c) 2011 Joris Degroote, Majid Hojjat, Electra Stavropoulou,
% Roland Wüchner, Kai-Uwe Bletzinger
%
% Licensed under the Non-Profit Open Software License version 3.0
% (NPOSL-3.0). You should have received a copy of the NPOSL-3.0 along with 
% this file. If not, see <http://www.opensource.org/licenses/NPOSL-3.0>
%
% Critertion for the convergence of the coupling iterations.

classdef Convergence < handle
    properties (SetAccess=private)
        reso=0.0;
        resk=0.0;
        k=0;
        k_tot=0;
        n=0;
        file;
    end
    properties (SetAccess=immutable)
        tol;
        k_min;
        k_e;
        n_e;
        small;
        output;
    end
    methods
        function C=Convergence(tol,k_min,k_e,n_e,small,output)
            C.tol=tol;
            C.k_min=k_min;
            C.k_e=k_e;
            C.n_e=n_e;
            C.small=small;
            C.output=output;
            if (C.output)
                C.file=fopen('Residual.txt','w');
            end
        end
        function delete(C)
            if (C.output)
                fclose(C.file);
            end
        end
        function add(C,r)
            C.k=C.k+1;
            C.k_tot=C.k_tot+1;
            C.resk=norm(r);
            if (C.k==1)
                C.reso=C.resk;
            end
            if (C.output)
                if ((C.is_satisfied() && (C.output==1)) || (C.output==2))
                    format='%4i %4i %11.4e\n';
                    data=[C.n C.k C.resk];
                    fprintf(C.file,format,data);
                    fprintf(format,data);
                end
            end
        end
        function conv=is_satisfied(C)
            if ((C.resk<max(C.reso*C.tol,C.small)) && (C.k>C.k_min))
                conv=true;
            else
                conv=false;
            end
            if ((C.k==C.k_e) && (~conv))
                error(['Coupling not converged after maximal number' ...
                    ' of iterations']);
            end
        end
        function display_average(C)
            if (C.output)
                disp(['Average number of coupling iterations = ' ...
                    num2str(C.k_tot/C.n_e)]);
            end
            C.k_tot=0;
        end
        function finish_primal(C)
            C.display_average();
        end
        function finish_dual(C)
            C.display_average();
        end
        function increase_time(C)
            C.k=0;
            C.n=C.n+1;
        end
        function decrease_time(C)
            C.k=0;
            C.n=C.n-1;
        end
    end
end