% Copyright (c) 2011 Joris Degroote, Majid Hojjat, Electra Stavropoulou,
% Roland Wüchner, Kai-Uwe Bletzinger
%
% Licensed under the Non-Profit Open Software License version 3.0
% (NPOSL-3.0). You should have received a copy of the NPOSL-3.0 along with 
% this file. If not, see <http://www.opensource.org/licenses/NPOSL-3.0>
%
% Calculation of the costfunction value and gradient.

classdef Costfunction < handle
    properties (Constant)
        path_simulation='DataSimulation.mat';
        path_reference='DataReference.mat';
    end
    properties (SetAccess=private)
        n=0;
        x;
        djds;
        pdRpds;
        scale=0.0;
        finished_primal=false;
        finished_dual=false;
    end
    properties (SetAccess=immutable)
        output;
        reference;
        range;
        m_e;
        s;
    end
    methods
        function C=Costfunction(m_e,s,output,reference)
            C.x=[];
            C.djds=[];
            C.pdRpds=[];
            
            C.output=output;
            C.reference=reference;
            C.range=5+2*m_e:4+3*m_e;
            C.m_e=m_e;
            C.s=s;
        end
        function add(C,x_f,x_s,pdR_fpds,pdR_spds)
            if (C.finished_primal)
                a=[x_f;x_s];
                b=C.pdRpds(:,length(C.s)*(C.n-1)+1:length(C.s)*C.n);
                C.djds=C.djds+a'*b;
            else
                C.x=[C.x [x_f;x_s]];
                C.pdRpds=[C.pdRpds [pdR_fpds;pdR_spds]];
            end
        end
        function j=calculate_value(C)
            if (C.finished_primal)
                j=sum(sum(C.x(C.range,:).^2))/C.scale;
            else
                error('Primal calculation not finished');
            end
        end
        function djds=calculate_gradient(C)
            if (C.finished_primal && C.finished_dual)
                djds=C.djds;
            else
                error('Primal and/or dual calculation not finished');
            end
        end
        function pdjpdx_f=calculate_gradient_f(C)
            pdjpdx_f=zeros(1,4+2*C.m_e);
        end
        function pdjpdx_s=calculate_gradient_s(C)
            pdjpdx_s=[2.0/C.scale*C.x(C.range,C.n)' zeros(1,C.m_e)];
        end
        function finish_primal(C)
            C.finished_primal=true;
            if (C.output)
                disp('Primal calculation finished');
            end
            data=C.x;
            save(C.path_simulation,'-double','data');
            if (C.reference)
                copyfile(C.path_simulation,C.path_reference);
                x_ref=data;
            else
                if (exist(C.path_reference,'file'))
                    x_ref=load(C.path_reference);
                    x_ref=x_ref.data;
                else
                    error('Reference calculation not found');
                end
            end
            C.scale=C.m_e*(C.n-1)*(max(max(x_ref(C.range,:)))...
                -min(min(x_ref(C.range,:))))^2;
            C.x=C.x-x_ref;
            C.djds=zeros(1,length(C.s));
        end
        function finish_dual(C)
            C.finished_dual=true;
            if (C.output)
                disp('Dual calculation finished');
            end
        end
        function increase_time(C)
            if (~C.finished_primal)
                C.n=C.n+1;
            else
                error('Primal calculation finished');
            end
        end
        function decrease_time(C)
            if (C.finished_primal && ~C.finished_dual)
                C.n=C.n-1;
            else
                error(['Primal calculation not finished and/or '...
                    'dual calculation finished']);
            end
        end
    end
end