% Copyright (c) 2011 Joris Degroote, Majid Hojjat, Electra Stavropoulou,
% Roland Wüchner, Kai-Uwe Bletzinger
%
% Licensed under the Non-Profit Open Software License version 3.0
% (NPOSL-3.0). You should have received a copy of the NPOSL-3.0 along with 
% this file. If not, see <http://www.opensource.org/licenses/NPOSL-3.0>
%
% Limited-memory Broyden-Fletcher-Goldfard-Shanno (L-BFGS) approximation
% of the Hessian's inverse without explicit storage of this matrix.

classdef HessianInverse < handle
    properties (SetAccess=immutable)
        m;
    end
    properties (SetAccess=private)
        X;
        Y;
        x_r;
        y_r;
    end
    methods
        function H=HessianInverse(m)
            H.m=m;
            H.X=[];
            H.Y=[];
            H.x_r=[];
            H.y_r=[];
        end
        function add(H,x,y)
            if (~isempty(H.x_r))
                dx=x-H.x_r;
                dy=y-H.y_r;
                H.X=[H.X dx];
                H.Y=[H.Y dy];
            end
            H.x_r=x;
            H.y_r=y;
            while (size(H.X,2)>H.m)
                H.X(:,1)=[];
                H.Y(:,1)=[];
            end
        end
        function Hy=mtimes(H,y)
            if (isempty(H.X))
                Hy=y;
            else
                q=y;
                j=size(H.X,2);
                a=zeros(j,1);
                r=zeros(j,1);
                for i=1:j
                    r(i)=1.0/(H.Y(:,i)'*H.X(:,i));
                end
                for i=j:-1:1
                    a(i)=r(i)*H.X(:,i)'*q;
                    q=q-a(i)*H.Y(:,i);
                end
                g=(H.X(:,j)'*H.Y(:,j))/(H.Y(:,j)'*H.Y(:,j));
                Hy=g*q;
                for i=1:j
                    b=r(i)*H.Y(:,i)'*Hy;
                    Hy=Hy+H.X(:,i)*(a(i)-b);
                end
            end
        end
    end
end