% Copyright (c) 2011 Joris Degroote, Majid Hojjat, Electra Stavropoulou,
% Roland Wüchner, Kai-Uwe Bletzinger
%
% Licensed under the Non-Profit Open Software License version 3.0
% (NPOSL-3.0). You should have received a copy of the NPOSL-3.0 along with 
% this file. If not, see <http://www.opensource.org/licenses/NPOSL-3.0>
%
% Parameter identification for flexible tube.

function Identify(m_e,s_e,optimiser,filename)
    Simulation(m_e,s_e,true);

    ObjectiveFunc=@(s)Simulation(m_e,s);
    s_o=zeros(length(s_e),1);
    
    switch lower(optimiser)
        case {'fminunc','matlab'}
            options=optimset('Display','iter','GradObj','on', ...
                'LargeScale','off','HessUpdate','bfgs','MaxIter',100, ...
                'TolX',1e-6,'TolFun',1e-6,'OutputFcn',@StoreConvergence);
            OptimiserFunc=@fminunc;
        case {'optimiser','custom'}
            options=optimset('TolX',1e-6,'TolFun',1e-6,'MaxIter',400, ...
                'MaxFunEvals',1000,'MaxNodes',15, ...
                'OutputFcn',@StoreConvergence);
            OptimiserFunc=@Optimiser;
        otherwise
            error('Optimiser selection failed');
    end
    [s_a,j,exitflag,~,djds]=OptimiserFunc(ObjectiveFunc,s_o,options);
    
    disp(['Error = ' num2str(max(abs(s_e-s_a))) ', Cost function = ' ...
        num2str(j) ', Optimality = ' num2str(max(abs(djds)))]);
    if (exitflag~=1)
        error('Identification failed');
    end
    
    function stop=StoreConvergence(s,optimValues,state)
        stop=false;
        iteration=optimValues.iteration;
        funcCount=optimValues.funccount;
        err=max(abs(s_e-s));
        fval=optimValues.fval;
        if (isempty(optimValues.stepsize))
            a=0;
        else
            a=optimValues.stepsize;
        end
        firstorderopt=optimValues.firstorderopt;
        fileformat='%i %i %e %e %e %e\n';
        
        if strcmp(state,'iter')
            file=fopen(filename,'a');
            fprintf(file,fileformat,iteration,funcCount,fval,a, ...
                firstorderopt,err);
            fclose(file);
        elseif strcmp(state,'init')
            file=fopen(filename,'w');
            fclose(file);
        end
    end
end