% Copyright (c) 2011 Joris Degroote, Majid Hojjat, Electra Stavropoulou,
% Roland Wüchner, Kai-Uwe Bletzinger
%
% Licensed under the Non-Profit Open Software License version 3.0
% (NPOSL-3.0). You should have received a copy of the NPOSL-3.0 along with 
% this file. If not, see <http://www.opensource.org/licenses/NPOSL-3.0>
%
% Matrix-free least-squares model.

classdef JacobianInverse < handle
    properties (SetAccess=private)
        xr=[];
        yr=[];
        V=[];
        W=[];
        T=[];
        n=0;
        finished_primal=false;
    end
    properties (SetAccess=immutable)
        small;
        reuse;
    end
    methods
        function M=JacobianInverse(small,reuse)
            M.small=small;
            M.reuse=reuse;
        end
        function add(M,x,y)
            if (~isempty(M.xr))
                M.V=[x-M.xr M.V];
                M.W=[y-M.yr M.W];
                M.T=[M.n M.T];
            end
            M.xr=x;
            M.yr=y;
        end
        function dy=predict(M,dx)
            singular=true;
            while (singular && (~isempty(M.V)))
                [Q,R]=qr(M.V,0);
                [v,i]=min(abs(diag(R)));
                if (v<M.small)
                    M.V(:,i)=[];
                    M.W(:,i)=[];
                    M.T(:,i)=[];
                    %warning('MATLAB:DataRemoved','Model removed data');
                else
                    singular=false;
                end
            end
            if (~isempty(M.V))
                c=R\(Q'*dx);
                dy=M.W*c;
            else
                if (isempty(M.yr))
                    error('Model cannot predict without any data');
                else
                    dy=zeros(size(M.yr));
                end
            end
        end
        function clear(M)
            M.xr=[];
            M.yr=[];
            if (M.finished_primal)
                old=find(M.T>M.n+M.reuse);
            else
                old=find(M.T<M.n-M.reuse);
            end
            if (~isempty(old))
                M.V(:,old)=[];
                M.W(:,old)=[];
                M.T(:,old)=[];
            end
        end
        function finish_primal(M)
            M.finished_primal=true;
            M.xr=[];
            M.yr=[];
            M.V=[];
            M.W=[];
            M.T=[];
        end
        function increase_time(M)
            M.n=M.n+1;
            M.clear();
        end
        function decrease_time(M)
            M.n=M.n-1;
            M.clear();
        end
        function r=ready(M)
            r=~isempty(M.V);
        end
    end
end