% Copyright (c) 2011 Joris Degroote, Majid Hojjat, Electra Stavropoulou,
% Roland Wüchner, Kai-Uwe Bletzinger
%
% Licensed under the Non-Profit Open Software License version 3.0
% (NPOSL-3.0). You should have received a copy of the NPOSL-3.0 along with 
% this file. If not, see <http://www.opensource.org/licenses/NPOSL-3.0>
%
% Run all tests and simulations.

if (~strcmp(version('-release'),'2010b'))
    input('WARNING: Only tested on R2010b. Press any key to continue.');
end

m_e=100;

disp('Test derivatives');
step=[-1 0 1];
pert=[1e-3 1e-4];
loca=[1 10 m_e+1];
TestDerivative(m_e,step,pert,loca);

disp('Test optimiser');
TestOptimiser();

disp('Simulate model (minimal)');
s_e=-ones(m_e+1,1);
[~,~,histmin]=Simulation(m_e,s_e,true,true,2);
save('HistoryMinimal.txt','-double','-ascii','histmin');

disp('Simulate model (nominal)');
s_e=zeros(m_e+1,1);
[~,~,histnom]=Simulation(m_e,s_e,true,true,2);
save('HistoryNominal.txt','-double','-ascii','histnom');

disp('Simulate model (maximal)');
s_e=ones(m_e+1,1);
[~,~,histmax]=Simulation(m_e,s_e,false,true,2);
save('HistoryMaximal.txt','-double','-ascii','histmax');

disp('Calculate gradient');
s_e=[sin(pi*(1:m_e)/m_e)';0.0];
Simulation(m_e,s_e,true,false,0);
s_e=zeros(m_e+1,1);
[~,djds]=Simulation(m_e,s_e,false,false,0);
save('Gradient.txt','-double','-ascii','djds');

disp('Identify parameters (smooth)');
s_e=[0.3+0.5*sin(pi*(1:m_e)/m_e)';0.7];
Identify(m_e,s_e,'Optimiser','ConvergenceSmooth.txt');

disp('Identify parameters (step)');
s_e=[-0.2*ones(floor(m_e*0.2),1);-0.6*ones(floor(m_e*0.5),1); ...
    -0.3*ones(m_e-floor(m_e*0.2)-floor(m_e*0.5),1);0.1];
Identify(m_e,s_e,'Optimiser','ConvergenceStep.txt');

disp('Analyse stability');
s_e=[sin(pi*(1:m_e)/m_e)';0.0];
Simulation(m_e,s_e,true,false,0);
s_e=zeros(m_e+1,1);
Stability(m_e,s_e);

disp('Convergence history');
s_e=zeros(m_e+1,1);
[~,~]=Simulation(m_e,s_e,false,false,2,0.01,1060,'QuasiNewton',0);
movefile('Residual.txt','ConvergenceQuasiNewton.txt');
[~,~]=Simulation(m_e,s_e,false,false,2,0.01,1060,'QuasiNewton',3);
movefile('Residual.txt','ConvergenceQuasiNewtonReuse.txt');
try
    [~,~]=Simulation(m_e,s_e,false,false,2,0.01,1060,'GaussSeidel');
catch exception
    disp(exception.message);
end
movefile('Residual.txt','ConvergenceGaussSeidel.txt');

disp('Make plots');
MakePlots();